/*********************************************************************
 *
 *                  BMS_UI_Main Header
 *
 *********************************************************************
 * FileName:        BMS_UI_Main.h
 * Processor:       PIC18F45K80
 * Compiler:        Microchip C18 v3.41
 * Company:         KIT - CN - IPE
 *
 * Author           Date         Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Ott W.           08.02.2016   Release
 *********************************************************************/

#ifndef BMS_UI_Main_H      // To avoid duplicate inclusion
#define BMS_UI_Main_H



/*********************************************************************
 *
 * Include Files
 *
 ********************************************************************/
#include <P18F25K80.h>
#include <delays.h>

#include "BMS_MISRA.h"
#include "BMS_UI_CAN.h"
#include "BMS_ECAN.h"
#include "BMS_EEPROM.h"
#include "BMS_UI_Measure.h"
#include "BMS_UI_ADC.h"





/*********************************************************************
 *
 * General purpose typedef's
 *
 ********************************************************************/
typedef union _CAN_CONFIG
{
  struct
  {
    uint8_t   SlaveNo;
    uint8_t   BRP;
    uint8_t   PROPSEG;
    uint8_t   PHSEG1;
    uint8_t   PHSEG2;
    uint8_t   SJW;
    uint8_t   PHSEG2_MODE;
    uint8_t   BUS_SAMPLE_MODE;
    uint8_t   WAKEUP_MODE;
    uint8_t   FILTER_MODE;
  };
} CAN_CONFIG;

typedef union _IRC_MSG_BUF
{
  struct
  {
    uint32_t  CAN_ID[9];
    uint8_t   CAN_DATA[9][8];
  };
} IRC_MSG_BUF;

typedef union
{
uint8_t EE_ADRESS[18];
    struct
    {
    uint8_t FLASH;
    uint8_t LFT1;
    uint8_t LFT2;
    uint8_t LFT3;
    uint8_t LFT4;
    uint8_t LIEF;
    uint8_t BAU1;
    uint8_t BAU2;
    uint8_t EE_REV;
    uint8_t FAM1;
    uint8_t FAM2;
    uint8_t GERAET;
    uint8_t LAEN;
    uint8_t SLAVE_ID;
    uint8_t LFT_1;
    uint8_t LFT_2;
    uint8_t BAU_1;
    uint8_t FAM_1;
    }EE_BYTE;
} SERIAL_NR_t;



/*********************************************************************
*
*                             Defines
*
*********************************************************************/
#define EV__TimeSlot           0x0001
#define EV__TimeSlot1000ms     0x0002
#define EV__Canbus             0x0004

#define CAN_Tx_ErrorCountMax   24u
#define CAN_Rx_ErrorCountMax   240u

#define On                      1
#define Off                     0

#define SlaveNo_VAL             0
#define BRP_VAL                 4
#define PROPSEG_VAL             1
#define PHSEG1_VAL              8
#define PHSEG2_VAL              6
#define SJW_VAL                 1
#define PHSEG2_MODE_VAL         1
#define BUS_SAMPLE_MODE_VAL     ECAN_BUS_SAMPLE_MODE_THRICE
#define WAKEUP_MODE_VAL         ECAN_WAKEUP_MODE_ENABLE
#define FILTER_MODE_VAL         ECAN_FILTER_MODE_DISABLE

#define STANDBY_MODE            0x01
#define CALIBRATION_MODE        0x02
#define RUN_MODE                0x04
#define ERROR_MODE              0x08
#define RESET_MODE              0x10
#define INITIALISATION_MODE     0x20

#define EE_DEFAULT_d            0xFF        // Default value EEPROM in debuggmode
#define EE_DEFAULT_f            0x00        // Default value EEPROM in flashmode
#define EE_LOAD                 0xAA        // EEPROM is written
#define EE_START_a              0x00        // EEPROM startadress
#define EE_308_a                0           // EEPROM startadress for frame 0x308
#define EE_309_a                8           // EEPROM startadress for frame 0x309

#define RUN_UI                  0x01
#define STOP_UI                 0x02
#define UNLOCK_SYSTEM_CAN       0x04
#define DISABLE_SYSTEM_CAN      0x08
#define SYSTEM_IDSEARCH         0x40
#define SYSTEM_SERNRPRG         0x80
#define SYSTEM_GETSERNR         0x0A

#define ERROR_OFFSET_p          20          // max. +Currentsense Offset +0.200A
#define ERROR_OFFSET_n         -20          // max. -Currentsense Offset -0.200A

#define SUPPLY_MIN              2372        // Limit min. Supply voltage in ADC value
#define SUPPLY_MAX              2622        // Limit max. Supply voltage in ADC value

#define NO_ERROR               0x00
#define OFFSET_ERROR           0x01         // Current offset error
#define SUPPLY_ERROR           0x02         // Supply voltage error
#define RELAIS_OFF             0x04         // Flag for Master Relais off

#define MAX_ALIVE               8

#define MAX_CURRENT             2500        // Max. Current is 25 A
#define MIN_CURRENT            -2500        // Min. Current is -25A





/*********************************************************************
*
*                        Globales
*
*********************************************************************/



/*********************************************************************
*
*                        Function Prototypes
*
*********************************************************************/
void  interrupt_at_low_vector ( void );
void  my_isr                  ( void );
void  ini                     (uint8_t SlaveStatus);
void  main                    ( void );



#endif
